#ifndef GAME_ENG_H
#define GAME_ENG_H

#include <sys/time.h> /*for stopwatch*/
#include "player.h"
#include "glheightfield.h"

#define GAME_DEFAULT_PORT 8423
#define GAME_MAX_CLIENTS 32

typedef struct {
	struct timeval query_time, reset_time;
} stopwatch_t;

typedef struct {
	char *servername;
	int port;

	int server_socket;
	int client_sockets[GAME_MAX_CLIENTS];
	int client_updated_me[GAME_MAX_CLIENTS];
	int client_game_cycle_delta[GAME_MAX_CLIENTS];
	player_t players[GAME_MAX_CLIENTS]; 
	int first_person_index;
	bullet_t *bullet_list_start;
	bullet_t *bullet_list_end;
	bullet_t *free_bullets;
	int num_bullets;
	glheight_t heightfield;
	float gravity;
	float player_height;
	int clamp_player_to_ground;
	unsigned int loop_counter, loop_packets, max_packet_size, max_packet_b, max_packet_s, max_packet_ps, max_packet_pps;
	/* unsigned int epoch_time, loop_time, net_time; */
	unsigned int fontList; /* should be in grapheng. */
	stopwatch_t sw;
	float	loop_time, net_in_time, net_out_time, event_time, draw_time, max_packet_wait;
	char *config_filename;
} game_t;

typedef struct {
	int packet_type;
	union {
		float floatdata[7];
		int intdata[7];
		char chardata[28];
	} data;
} game_packet_t;



int game_clear(game_t *gamedata);				/* clear player list, alloc bullets, etc*/
int game_init(game_t *gamedata); 			/* start server process */
int game_connect(game_t *gamedata, char *ipaddr, int portno); 	/* connect to server */
int game_join(game_t *gamedata, char *name);	/* Add me to player list */
int game_disconnect(game_t *gamedata);			/* disconnect from server */
int game_quit(game_t *gamedata);				/* kill server process */

int game_fileconfig(game_t *gamedata);	/* configure game variables via config file */

int game_submit(game_t *gamedata, player_t *player); 		/* keyboard/mouse submission */
int game_update(game_t *gamedata); /* get new states */

int game_send(game_t *gamedata, game_packet_t *packet); /* sent packet to server */
int game_recv(game_t *gamedata, char *buffer, int buflen); /* get packets from server */

int clients_update(game_t *gamedata); /* server side of update */

/* tenths of a second */
float stopwatch(stopwatch_t *sw, int function);
                /*0 reset */
                /*1 query time since reset */
                /*2 query time since last query */



#define GAME_KILL 		 86
#define GAME_SUBMIT 		 80
#define GAME_JOIN 		101
#define GAME_PLAYER_POSITION 	102
#define GAME_PLAYER_NAME 	103
#define GAME_PLAYER_EVENT 	104
#define GAME_BULLET_POSITION 	201
#define GAME_BULLET_EVENT	202
#define GAME_UPDATE_END		999

#define GAME_MOUSE_ACCEL	1.0


#endif
