#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h> /* for sleep */

#include "glx_base.h"
#include "player.h"
#include "gameeng.h"
#include "grapheng.h"

void usage(char *progname) {
	printf("Usage: %s [-pure] [-connect AAA.BBB.CCC.DDD] [-port PPPP] [-fs] [-h] [-config file]\n", progname);
	printf("	-pure: start a pure server, no graphics\n");
	printf("	-connect: supply dot notation ip address\n");
	printf("	-port: supply server port number\n");
	printf("	-fs: start in fullscreen, not windowed, mode\n");
	printf("	-h: this message\n");
	printf("	-config : select configuration file. Default: config.punter\n");
}

int main(int argc, char **argv) {
	int i, slen, idx, portno;
        BOOL fullscreen, pure_server;
	char *server_address,
		*lochost        = "localhost", 
		*default_config = "config.punter";
	char valid_option;
	game_t game;
                                                                                
        fullscreen = False;
        pure_server = False;
	portno = 0;
	server_address = NULL;

	game_clear(&game);
	game.config_filename = malloc(sizeof(char) * strlen(default_config));
	strcpy(game.config_filename, default_config);
                                                                                
	/* parse command line options */
        for (i = 1; i < argc ; i++) {
		valid_option = 0x00;
                if (strcmp("-fs", argv[i]) == 0) {
                        fullscreen = True;
			valid_option = 0x01;
		}
                if (strcmp("-pure", argv[i]) == 0) {
                        pure_server = True;
			printf("Starting pure server\n");
			valid_option = 0x01;
		}
		if (strcmp("-port", argv[i]) == 0) {
			i++;
			if (i >= argc) { /* port requires additional argument */
				usage(argv[0]);
				exit(1);
			}
			slen = strlen(argv[i]);
			for (idx = 0; idx < slen ; idx++) 
					portno = portno * 10 + argv[i][idx] - '0';
			valid_option = 0x01;
		}
		if (strcmp("-connect", argv[i]) == 0) {
			i++;
			if (i >= argc) { /* connect requires additional argument */
				usage(argv[0]);
				exit(1);
			}
			slen = strlen(argv[i]);
			server_address = malloc(sizeof(char) * (slen + 1 ));
			strcpy(server_address, argv[i]);
			fprintf(stderr, "Using server %s \n", server_address);
			valid_option = 0x01;
		} /* -connect */
		if (strcmp("-config", argv[i]) == 0) {
			free(game.config_filename);
			i++;
			if (i >= argc) { /* config requires additional argument */
				usage(argv[0]);
				exit(1);
			}
			slen = strlen(argv[i]);
			game.config_filename = malloc(sizeof(char) * (slen + 1 ));
			strcpy(game.config_filename, argv[i]);
			fprintf(stderr, "Using config file %s \n", game.config_filename);
			valid_option = 0x01;
		} /* -config */

                if (strcmp("-h", argv[i]) == 0) {
			usage(argv[0]);
			exit(0);
		}

		if (valid_option == 0) {
			fprintf(stderr, "Unknown Option %s\n", argv[i]);
			usage(argv[0]);
			exit(1);
		}
        }

	if (portno == 0) {
		fprintf(stderr, "No Port Specified.  Using default port %d\n", GAME_DEFAULT_PORT);
		portno = GAME_DEFAULT_PORT;
	}

	game.port = portno;
	/* either start or connect to a server */
	if (server_address == NULL) { /* no server specified, so create one */
		game_init(&game); 
		/* let it start up */
		sleep(2);
		server_address = lochost;
	}
	
	if ( pure_server == False ) {
		game_fileconfig(&game);
        	createGLWindow("Punter", 1280, 1024, 24, fullscreen);

		 game_connect(&game, server_address, portno);  
        	/* createGLWindow("NeHe's GLX Base Code", 640, 480, 24, fullscreen); */
		game_join(&game, "Human");

                                                                                
        	/* Initialize the OpenGL GLX Scene */
        	gr_init(&game);
                                                                                
        	/* Run the OpenGL GLX Scene */
        	mainLoop(&game);
                                                                                
        	/* Deinitialize OpenGL GLX Scene and clean up anything we have
         	* initialized
        	cleanGLScene();
         	*/
     		/* Kill the OpenGL GLX Scene */
        	killGLWindow();
	}
                                                                                
        return 0;
}
                                         
