#include <stdio.h>
#include <string.h>
#include <math.h>
#include "player.h"


void player_copy(player_t *src, player_t *dst) {
/*	src->location[0] = dst->location[0] ;
	src->location[1] = dst->location[1] ;
	src->location[2] = dst->location[2] ;
	src->heading[0] = dst->heading[0] ;
	src->heading[1] = dst->heading[1] ;
	src->velocity = dst->velocity ;
	src->player_id = dst->player_id ;
	src->mouse_buttons = dst->mouse_buttons ;
	src->key_actions = dst->key_actions ;
	src->mouse_delta[0] = dst->mouse_delta[0] ;
	src->mouse_delta[1] = dst->mouse_delta[1] ;
	src->state = dst->state ;
	src->state_counter = dst->state_counter ; */

	memcpy(dst, src, sizeof(player_t));
}

void player_clear(player_t *p) {

	p->location[0] = 0.0;
	p->location[1] = 0.0;
	p->location[2] = 0.0;
	p->heading[0] = 0.0;
	p->heading[1] = 0.0;
	p->velocity = 0.0;
	p->player_id = -1;
	p->mouse_buttons = -1;
	p->key_actions = -1;
	p->mouse_delta[0] = 0;
	p->mouse_delta[1] = 0;
	p->state = 0;
	p->state_counter = 0;
	p->hit_points = 100;
	p->status = PLAYER_STATUS_INACTIVE;
	p->player_name = NULL;
	p->height = 0.0;
}

void player_print(player_t *p) {
	char *name, tmp[] = "UNKNOWN";
	
	if (p->player_name == NULL) {
		printf("null name, using default\n ");
		name = tmp;
	} else {
		printf("NOT null name, using %s\n ", p->player_name);
		name = p->player_name;
	}

	printf("Player %s\n Location: %3.2f %3.2f %3.2f heading: %3.2f %3.2f\n",
		name, p->location[0], p->location[1], p->location[2],
		p->heading[0], p->heading[1]);
	printf("	Velocity: %3.2f, Player ID: %d, Mouse[b:%d, d:%d,%d\n",
		p->velocity, p->player_id, p->mouse_buttons, p->mouse_delta[0],
		p->mouse_delta[1]);
	printf("	Status: %d   Keyactions: %d  State: %d State cntr:%d\n",
		p->status, p->key_actions, p->state, p->state_counter );
}

 
void player_shoot(player_t *p, bullet_t *b) {
	float muzzle_length = 14.0;

	b->heading[0] =  p->heading[0];
	b->heading[1] =  p->heading[1];
	/* slow is pretty. 0.04 bye */
	b->velocity = 0.3;
	/* I would like to displace the bullet to be
		in front of player, not start at the player origin.
	*/
        b->location[2] = p->location[2] - 
			p->cosxz * p->cosupness * muzzle_length;
        b->location[1] = p->location[1] + p->height - 
			p->upness * muzzle_length;
        b->location[0] = p->location[0] - 
			p->sinxz * p->cosupness * muzzle_length;
	b->origin[0] = b->location[0];
	b->origin[1] = b->location[1];
	b->origin[2] = b->location[2];

	b->type = BULLET_TYPE_ROCKET;
}


void bullet_move(bullet_t *b, float velocity_factor) {
        float cosxz, sinxz, upness, cosupness;

  	switch (b->type) {
        	case BULLET_TYPE_ROCKET:
        		cosxz = cos(deg2rad(b->heading[1]));
        		sinxz = -sin(deg2rad(b->heading[1]));
        		/* neck up/down */
        		cosupness = cos(deg2rad(b->heading[0])) * b->velocity * velocity_factor;
        		upness   = -sin(deg2rad(b->heading[0])) * b->velocity * velocity_factor;
		
        		b->location[2] -= cosxz * cosupness;
        		b->location[1] -= upness;
        		b->location[0] -= sinxz * cosupness;
			break;

		case BULLET_TYPE_EXPLOSION:
        		b->heading[0] += 0.03 * velocity_factor;
			break;
		case BULLET_TYPE_SHIP_EXPLOSION:
        		b->heading[0] += 0.1 * velocity_factor;
			break;
	}
			
}

float bullet_length_2(bullet_t *b) {
	float r2;
	r2 = (
		( b->location[0] - b->origin[0] ) *
		( b->location[0] - b->origin[0] ) +
		( b->location[1] - b->origin[1] ) *
		( b->location[1] - b->origin[1] ) +
		( b->location[2] - b->origin[2] ) *
		( b->location[2] - b->origin[2] )
	);
	return r2;

}
int player_bbox(bullet_t *b, player_t *p) {
	/* float azimuth, xzangle; */

	return 0;
}

int bullet_hit_player(bullet_t *b, player_t *p) {
	/* hmmm.... */
	float bullet_vs_player_radius_squared;
	int hit;

	hit = 0;

	bullet_vs_player_radius_squared = 
		( b->location[0] - p->location[0] ) *
		( b->location[0] - p->location[0] ) +
		( b->location[1] - p->location[1] ) *
		( b->location[1] - p->location[1] ) +
		( b->location[2] - p->location[2] ) *
		( b->location[2] - p->location[2] );
	
	if (bullet_vs_player_radius_squared < 9.0) { 
		/* apply bbox test */
		p->hit_points -= 1;
		if (p->hit_points <= 0) { 
			b->type = BULLET_TYPE_SHIP_EXPLOSION; /* change to explosion */
			p->status = PLAYER_STATUS_DEAD;
		} else {
			b->type = BULLET_TYPE_EXPLOSION; /* change to explosion */
		}
		b->heading[0] = 0.1; /* since it's changing to an explosion, set explosion variable */
		hit = 1;
	}
	return hit;
}
	
void player_calc_orientation(player_t *p) 
{
	float velocity, azimuth, xzangle;

        azimuth = deg2rad(p->heading[0]);
        xzangle = deg2rad(p->heading[1]);
	velocity = p->velocity;

        p->cosxz =  cos(xzangle);
        p->sinxz = -sin(xzangle);
        p->rtc   = -p->sinxz * velocity;
        p->rts   = p->cosxz * velocity;
        p->cosupness = cos(azimuth) * velocity;
        p->upness = -sin(azimuth) * velocity;
}

