#ifndef PLAYER_H
#define PLAYER_H

#include "quaternion.h"

#define PLAYER_KEY_FORWARD	0x0001
#define PLAYER_KEY_BACKWARD	0x0002
#define PLAYER_KEY_LEFT		0x0004
#define PLAYER_KEY_RIGHT	0x0008
#define PLAYER_KEY_RUN	 	0x0010
#define PLAYER_KEY_JUMP	 	0x0020
#define PLAYER_KEY_USE	 	0x0040
#define PLAYER_KEY_SPECIAL 	0x0080
#define PLAYER_KEY_STRAFE_LEFT	0x0100
#define PLAYER_KEY_STRAFE_RIGHT	0x0200

#define PLAYER_MOUSE_BUTTON_0	0x0001
#define PLAYER_MOUSE_BUTTON_1	0x0002
#define PLAYER_MOUSE_BUTTON_2	0x0004

#define PLAYER_STATUS_INACTIVE	0x0000
#define PLAYER_STATUS_ALIVE	0x0001
#define PLAYER_STATUS_DEAD	0x0002

#define BULLET_TYPE_ROCKET	0x0001
#define	BULLET_TYPE_EXPLOSION	0x0002
#define	BULLET_TYPE_SHIP_EXPLOSION	0x0003

typedef struct player_tag {
	v3 location;
	v2 heading;
	v1 velocity;
	v1 height;

	int player_id;
	char *player_name;


	int status;
	int mouse_buttons; /* up to 32 buttons */
	int key_actions; /* up to 32 keys */
	int mouse_delta[2];

	int hit_points;
	int state;
	int state_counter;

	/* cached orientation */
	float cosxz, sinxz, upness, cosupness, rtc, rts;

	struct player_tag *next;
} player_t;

typedef struct bullet_tag {
	v3 origin;
	v3 location;
	v2 heading;
	v1 velocity;

	int type;
	int state;
	int state_counter;

	struct bullet_tag *next;
} bullet_t;

void player_copy(player_t *src, player_t *dst);
void player_print(player_t *p);
void player_clear(player_t *p);
void player_calc_orientation(player_t *p); /* cache sin/cos of orientation*/
void player_shoot(player_t *p, bullet_t *b);
void bullet_move(bullet_t *b, float velocity_factor);
int bullet_hit_player(bullet_t *b, player_t *p);
float bullet_length_2(bullet_t *b);
	
#endif
