#include <stdio.h>
/* #include <GL/glut.h> */
#include <math.h>
#include <pam.h>
#include <GL/glu.h>
#include "ppmgl.h"

void PPMGLerror(char *errstr) {
	GLenum err;

	if ( ( err = glGetError()) != GL_NO_ERROR ) {
		fprintf(stderr, "GL ERROR: %s : %s \n", errstr, gluErrorString(err));
		exit(1);
	}
}

void error_abort(char *message1, char *message2) {
		fprintf(stderr, "ERROR: %s : %s \n",message1, message2);
		exit(1);
}

GLubyte *get_bmp_pixels(char *fname, int *xresp, int *yresp, int bpp, GLfloat alpha) {
	GLubyte *image_map;
	short int twobytes, data_offset;
	int fourbytes, img_size;
	FILE *img_f;

        /* read bmp file */
        if ((img_f = fopen(fname, "rb"))) {
		/* bitmap file header */
		fread(&twobytes, sizeof(short int), 1, img_f);
		if (twobytes != 19778 ) error_abort("not BMP file", fname);
		fread(&fourbytes, sizeof(int), 1, img_f);
		printf("Size of bmp file: %d\n", fourbytes);
		fread(&twobytes, sizeof(short int), 1, img_f);
		if (twobytes != 0 ) error_abort("not zero for bfReserved1:", fname);
		fread(&twobytes, sizeof(short int), 1, img_f);
		if (twobytes != 0 ) error_abort("not zero for bfReserved2:", fname);
		fread(&fourbytes, sizeof(int), 1, img_f);
		printf("offset of bmp data: %d\n", fourbytes);
		data_offset = fourbytes;

		/* bitmap info header */
		fread(&fourbytes, sizeof(int), 1, img_f);
		printf("size of bitmap info header(40): %d\n", fourbytes);
		fread(&fourbytes, sizeof(int), 1, img_f);
		printf("width of bitmap: %d\n", fourbytes);
		*xresp = fourbytes;
		fread(&fourbytes, sizeof(int), 1, img_f);
		printf("height of bitmap: %d\n", fourbytes);
		*yresp = fourbytes;
		fread(&twobytes, sizeof(short int), 1, img_f);
		if (twobytes != 1 ) error_abort("not one for planes:", fname);
		fread(&twobytes, sizeof(short int), 1, img_f);
		if (twobytes != 24 ) error_abort("not 24 bitplanes!", fname);
		fread(&fourbytes, sizeof(int), 1, img_f);
		if (fourbytes != 0 ) error_abort("not 0 for compression!", fname);
		fread(&fourbytes, sizeof(int), 1, img_f);
		printf("size of bitmap: %d\n", fourbytes);
		img_size = fourbytes;
		fread(&fourbytes, sizeof(int), 1, img_f);
		printf("xpixels per meter: %d\n", fourbytes);
		fread(&fourbytes, sizeof(int), 1, img_f);
		printf("ypixels per meter: %d\n", fourbytes);
		fread(&fourbytes, sizeof(int), 1, img_f);
		printf("number of colors in colormap: %d\n", fourbytes);
		fread(&fourbytes, sizeof(int), 1, img_f);
		printf("important color in colormap: %d\n", fourbytes);

		/* actual pixel data */
		/* 24 bpp reader: */
                image_map = (GLubyte *) 
                        malloc( sizeof(GLubyte) * *xresp * *yresp * 4);
		twobytes = fread(image_map, img_size, 1, img_f);
		if (twobytes != 1) {
			printf("Read: %d, should have read: %d\n", 
				twobytes ,img_size);
			if (feof(img_f)) printf("end of file detected \n");
			if (ferror(img_f)) printf("error of file detected \n");
			 	error_abort("short read in pixels", fname);
		}
		fclose(img_f);
	} else {
                fprintf(stderr, "Could not open %s for reading!\n", fname);
                exit(1);
	}

	return (image_map);
}
		
/* read pixel data from file, return x,y resolution and byte array */
GLubyte *get_ppm_pixels(char *fname, int *xresp, int *yresp, int bpp, GLfloat alpha) {

	GLubyte *image_map, rb, bb, gb;
	FILE *img_f;

        int width, height,  i, j, offset;
	unsigned int maxval;
        pixel **pnmrows, a_pixel, *pixel_row;

        /* read ppm file */

        if ((img_f = fopen(fname, "r"))) {
                pnmrows = ppm_readppm(img_f, &width, &height, &maxval);
                /* printf("Image %s is %d by %d with max %d\n", 
			fname, width, height, maxval); */
                image_map = (GLubyte *) 
                        malloc( sizeof(GLubyte) * width * height * 4);
                for ( j = 0 ; j < height ; j++)
                        for ( i = 0 ; i < width ; i++) {
                                offset = ( width *  j + i ) ;
                                pixel_row = pnmrows[height - j - 1];  
                                /* pixel_row = pnmrows[j];  */
                                a_pixel = pixel_row[i];
                                rb = PPM_GETR(a_pixel);
                                gb = PPM_GETG(a_pixel);
                                bb = PPM_GETB(a_pixel);
                                image_map[offset * 4 + 0] = rb;
                                image_map[offset * 4 + 1] = gb;
                                image_map[offset * 4 + 2] = bb;
				if ( (rb & bb & gb) == 0xff )
                                	image_map[offset * 4 + 3] = 0;
				else
                                	image_map[offset * 4 + 3] = 255;
                        }
		fclose(img_f);
        } else {
                fprintf(stderr, "Could not open %s for reading!\n", fname);
                exit(1);
        }
	*xresp = width;
	*yresp = height;

	return (image_map);
}

/* create texture opject from filename */
void makePPMTexture(char *map_filename) {
	GLubyte *imagepixels;
	int xres, yres;

	imagepixels = get_ppm_pixels(map_filename, &xres, &yres, 3, 0.0);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, xres, yres, 
			0, GL_RGBA, GL_UNSIGNED_BYTE, imagepixels);
	free(imagepixels);
	PPMGLerror("glTexImage2D");
}

void makeBMPTexture(char *bmp_filename) {
	GLubyte *imagepixels;
	int xres, yres;

	imagepixels = get_bmp_pixels(bmp_filename, &xres, &yres, 3, 0.0);
	printf("xres %d, yres %d, loc %lu %x %x %x %x %x\n", xres, yres, (long unsigned int) imagepixels,
		imagepixels[0],
		imagepixels[1],
		imagepixels[2],
		imagepixels[3],
		imagepixels[4]);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, xres, yres, 
			0, GL_BGR, GL_UNSIGNED_BYTE, imagepixels);
	PPMGLerror("glTexImage2D");
	free(imagepixels);
}
