#include <stdio.h>
#include <stdlib.h>
#include "quaternion.h"


#include <math.h>

#ifdef COLUMN_ORDER

#define QMidx(n,i,j) ((j) * (n) + (i))
/* for 4x4 matrices */
#define QM11 0
#define QM21 1
#define QM31 2
#define QM41 3
#define QM12 4
#define QM22 5
#define QM32 6
#define QM42 7
#define QM13 8
#define QM23 9
#define QM33 10
#define QM43 11
#define QM14 12
#define QM24 13
#define QM34 14
#define QM44 15
/* for 3x3 matrices */
#define Qm11 0
#define Qm12 1
#define Qm13 2
#define Qm21 3
#define Qm22 4
#define Qm23 5
#define Qm31 6
#define Qm32 7
#define Qm33 8

#else /* row order */
#define QMidx(n,i,j) ((i) * (n) + (j))

/* for 4x4 matrices */
#define QM11 0
#define QM12 1
#define QM13 2
#define QM14 3
#define QM21 4
#define QM22 5
#define QM23 6
#define QM24 7
#define QM31 8
#define QM32 9
#define QM33 10
#define QM34 11
#define QM41 12
#define QM42 13
#define QM43 14
#define QM44 15

/* for 3x3 matrices */
#define Qm11 0
#define Qm21 1
#define Qm31 2
#define Qm12 3
#define Qm22 4
#define Qm32 5
#define Qm13 6
#define Qm23 7
#define Qm33 8

#endif

#ifndef M_PI
#define M_PI 3.141592654
#endif

/* leave result in q1 */
qFloat  * qnew_acc(quaternion q1, qFloat theta, qFloat *u)
{
	qFloat s = sin(theta * 0.5);
	q1[0] = cos(theta * 0.5);
	q1[1] = u[0] * s;
	q1[2] = u[1] * s;
	q1[3] = u[2] * s;
	return q1;
}
qFloat *  qmul_acc(quaternion q1, quaternion q2)
{
	qFloat d, a, b, c;
	d = q1[0];
	a = q1[1];
	b = q1[2];
	c = q1[3];
	q1[0] = d * q2[0] - a * q2[1] - b * q2[2] - c * q2[3];
	q1[1] = d * q2[1] + a * q2[0] + b * q2[3] - c * q2[2];
	q1[2] = d * q2[2] + b * q2[0] + c * q2[1] - a * q2[3];
	q1[3] = d * q2[3] + c * q2[0] + a * q2[2] - b * q2[1];
	return q1;
}
qFloat *  qadd_acc(quaternion q1, quaternion q2)
{
/*	q1[0] = q1[0] + q2[0];
	q1[1] = q1[1] + q2[1];
	q1[2] = q1[2] + q2[2];
	q1[3] = q1[3] + q2[3]; */
	vec_add(q1, q2, 4);
	return q1;
}
qFloat *  qcopy_acc(quaternion dest, quaternion src)
{
	dest[0] = src[0];
	dest[1] = src[1];
	dest[2] = src[2];
	dest[3] = src[3];
	return dest;
}

qFloat   * qinv_acc(quaternion q) {

	q[0] =  q[0];
	q[1] = -q[1];
	q[2] = -q[2];
	q[3] = -q[3];

	return q;
}

qFloat   * qget_acc(quaternion q, qFloat *theta, qFloat *u) {

	qFloat s;

	*theta = acosf(q[0]) * 2.0;
	s = sin(*theta * 0.5);
	/* q[0] = c, s * s + c * c = 1, s = sqrt(1 - c * c). faster? */
	if (s != 0.0)
		s = 1.0 / s;
	else
		s = 1.0;
	u[0] = q[1] * s;
	u[1] = q[2] * s;
	u[2] = q[3] * s;

	return theta;
}

/* return result in new */
qFloat * qnew(qFloat theta, qFloat u[3])
{
	qFloat s, *r;

	r = (qFloat *) malloc(sizeof(quaternion));

	s = sin(theta * 0.5);
	r[0] = cos(theta * 0.5);
	r[1] = u[0] * s;
	r[2] = u[1] * s;
	r[3] = u[2] * s;
	return r;
}

qFloat * qmul(quaternion q1, quaternion q2)
{
	qFloat *q;

	q = (qFloat * ) malloc(sizeof(quaternion));
	qcopy_acc(q, q1);
	qmul_acc(q, q2);

	return q;
}

qFloat * qadd(quaternion q1, quaternion q2)
{
	qFloat *q;

	q = (qFloat * ) malloc(sizeof(quaternion));

	vec_copy(q, q1, 4);
	vec_add(q, q2, 4);
	/*q[0] = q1[0] + q2[0];
	q[1] = q1[1] + q2[1];
	q[2] = q1[2] + q2[2];
	q[3] = q1[3] + q2[3]; */

	return q;
}

qFloat *  qcopy(quaternion q2)
{
	qFloat *q1;

	q1 = (qFloat * ) malloc(sizeof(quaternion));
	vec_copy(q1, q2, 4);

	return q1;
}

/*util*/
void qprint(quaternion q)
{
	qFloat theta, u[3];

	qget_acc(q, &theta, u);
	printf("Quaternion: [%5.2f; %5.2f, %5.2f, %5.2f ] /Rot: %5.2f via [%5.2f, %5.2f, %5.2f] len: %3.2f\n",
		q[0], q[1], q[2], q[3],
		theta / M_PI * 180.0, u[0], u[1], u[2], sqrt(vec_lensq(u,3)));
	
}
qFloat   * qinv(quaternion q) {
	qFloat *qinv;

	qinv = (qFloat * ) malloc(sizeof(quaternion));
	qinv[0] =  q[0];
	qinv[1] = -q[1];
	qinv[2] = -q[2];
	qinv[3] = -q[3];
	return qinv;
}


/* v = q*v*q^-1 */
qFloat   * qrotate(quaternion q, qFloat *v) {
	quaternion qinv, qcpy, qtmp;

	qinv[0] = q[0];
	qinv[1] = -q[1];
	qinv[2] = -q[2];
	qinv[3] = -q[3];

	qtmp[0] = 0.0;
	qtmp[1] = v[0];
	qtmp[2] = v[1];
	qtmp[3] = v[2];

	qcopy_acc(qcpy, q);

	qmul_acc(qcpy, qtmp);
	qmul_acc(qtmp, qinv);
	v[0] = qtmp[1];
	v[1] = qtmp[2];
	v[2] = qtmp[3];

	return v;
}
                                       
qFloat   * q_to_matrix(quaternion q, m16 m) {
	qFloat dd, aa, bb, cc, tab, tac, tad,
		tbc, tbd, tcd;
	dd = q[0] * q[0];
	aa = q[1] * q[1];
	bb = q[2] * q[2];
	cc = q[3] * q[3];

	tab = q[1] * q[2] * 2.0;
	tac = q[1] * q[3] * 2.0;
	tad = q[1] * q[0] * 2.0;

	tbc = q[2] * q[3] * 2.0;
	tbd = q[2] * q[0] * 2.0;

	tcd = q[3] * q[0] * 2.0;

	m[ 0] = dd + aa - bb - cc;
	m[ 1] = tab + tcd;
	m[ 2] = tac - tbd;
	m[ 3] = 0.0;
	m[ 4] = tab - tcd;
	m[ 5] = dd - aa + bb - cc;
	m[ 6] = tbc + tad;
	m[ 7] = 0.0;
	m[ 8] = tac + tbd;
	m[ 9] = tbc - tad;
	m[10] = dd - aa - bb + cc;
	m[11] = 0.0;
	m[12] = 0.0;
	m[13] = 0.0;
	m[14] = 0.0;
	m[15] = 1.0;

	return m;
} 

/* treat as 3x3 matrix */
qFloat   * matrix_to_q(quaternion q, m16 m)
{
	qFloat trace, factor4;
	int i;

	i = 0;
	trace = m[0] + m[5] + m[10];
	if 	  (m[0]  > trace) {
		i = 1;
	} else if (m[5]  > trace) {
		i = 2;
	} else if (m[10] > trace) {
		i = 3;
	}
	switch (i) {
		case 0:
			q[0] = 0.5 * sqrt(trace + 1.0);
			factor4 = 0.25 / q[0];
			q[1] = (m[QM32] -m[QM23]) * factor4;
			q[2] = (m[QM13] -m[QM31]) * factor4;
			q[3] = (m[QM21] -m[QM12]) * factor4;
			break;
		case 1:
			q[1] = 0.5 * sqrt(2.0 * m[QM11] - trace + 1.0);
			factor4 = 0.25 / q[1];
			q[0] = (m[QM32] -m[QM23]) * factor4;
			q[2] = (m[QM21] -m[QM12]) * factor4;
			q[3] = (m[QM13] -m[QM31]) * factor4;
			break;
		case 2:
			q[2] = 0.5 * sqrt(2.0 * m[QM22] - trace + 1.0);
			factor4 = 0.25 / q[2];
			q[0] = (m[QM13] -m[QM31]) * factor4;
			q[1] = (m[QM21] -m[QM12]) * factor4;
			q[3] = (m[QM32] -m[QM23]) * factor4;
			break;
		case 3:
			q[3] = 0.5 * sqrt(2.0 * m[QM33] - trace + 1.0);
			factor4 = 0.25 / q[3];
			q[0] = (m[QM21] -m[QM12]) * factor4;
			q[1] = (m[QM13] -m[QM31]) * factor4;
			q[2] = (m[QM32] -m[QM23]) * factor4;
			break;
	}
	return q;
}


qFloat   * lerp(qFloat *dest, qFloat *start, qFloat *end, qFloat alpha, qFloat len)
{
	int i;
	
	/*for (i = 0 ; i < len ; i++) dest[i] =
			start[i] * (1.0 - alpha) + end[i] * alpha; */
	for (i = 0 ; i < len ; i++) dest[i] = 
			start[i] + (end[i] - start[i]) * alpha;
			/* one multiply */
	return dest;
}
                                                                                
qFloat   * slerp_precomp(qFloat *sphi, qFloat *x, qFloat *y, int len)
{
	qFloat c;
	qFloat v[4], cx[4], *pv, *pcx;
	int alloc;
	
	if (len > 4) { 
		pv = (qFloat *) malloc(sizeof(qFloat) * len);
		pcx = (qFloat *) malloc(sizeof(qFloat) * len);
		alloc = 1;
	} else {
		pv = v;
		pcx = cx;
		alloc = 0;
	}
	/* c = x.y     */
	c = vec_dot(x, y, len);
	sphi[2] = c;
	/* v = y - cx; */
	vec_copy(pv, y, len);
	if (c < 0.0) {
		vec_scalar_mul(-1.0, pv ,len);
		c = -c;
	}
	vec_copy(pcx, x, len);
	vec_scalar_mul(-c, pcx, len);
	vec_add(pv, pcx, len);
	sphi[0] = sqrt( vec_lensq(pv, len)); /* sin(phi) */
	sphi[1] = atan2f(sphi[0], c);       /* phi = atan(s/c) */
	if (alloc) {
		free(pv);
		free(pcx);
	}
	return sphi;
}
qFloat   * slerp(qFloat *sphi, quaternion qdest, quaternion q1,quaternion q2, qFloat alpha)
{
	qFloat p[3], sphiinv, factor1, factor2;
	quaternion y;

	if (sphi == NULL) {
		slerp_precomp( p, q1, q2, 4);
		sphi = p;
	}

	/* epsilon test for unstable sin() */
	if (sphi[0] * sphi[0] < 0.00000008) {
		factor1 = (1.0 - alpha);
		factor2 = alpha;
	} else {
		sphiinv = 1.0 / sphi[0];
		factor1 = sin((1.0 - alpha) * sphi[1]) * sphiinv;
		factor2 = sin(       alpha  * sphi[1]) * sphiinv;
	}
	/* qdest = factor1 * q1 + factor2 * q2 */
	/*printf("Phi: %3.2f Factor1: %3.2f factor2: %3.2f ", sphi[1], factor1, factor2); */
	vec_copy(qdest, q1, 4);
	vec_scalar_mul(factor1, qdest, 4);
	vec_copy(y, q2, 4);
	if (sphi[2] >= 0.0)
		vec_scalar_mul(factor2, y, 4);
	else
		vec_scalar_mul(-factor2, y, 4);
	vec_add(qdest, y, 4);

	return qdest;
}

qFloat   vec_lensq(qFloat *v, int len)
{
	qFloat d;
	int i;

	d = 0.0;
	for (i = 0 ; i < len ; i++) d += v[i] * v[i];

	return d;
}

qFloat vec_dot(qFloat *v1, qFloat *v2, int len)
{
	qFloat accum;
	int i;

	accum = 0.0;
	for (i = 0; i < len ; i++) accum += v1[i] * v2[i];

	return accum;
}

qFloat   *vec_scalar_mul(qFloat d, qFloat *v, int len)
{
	int i;

	for (i = 0 ; i < len ; i++) v[i] *= d;
	return v;
}

qFloat   * vec_unit(qFloat *v, int len)
{
	qFloat d;
#define DELTAL 0.999
#define DELTAH 1.001
	d = vec_lensq(v, len);
	if ((d > DELTAL) && (d < DELTAH)) /* this is very close to normal */
		return v;
	if (d == 0.0)
		d = 1.0;
	else
		d = 1.0 / sqrt(d);
	vec_scalar_mul(d, v, len);

	return v;
}

qFloat   * vec_add(qFloat *v1, qFloat *v2, int len) {
	int i;
	for (i = 0 ; i < len ; i++) v1[i] += v2[i];
	return v1;
}

qFloat   * vec_copy(qFloat *dst, qFloat *src, int len)
{
	int i;
	for (i = 0 ; i < len ; i++) dst[i] = src[i];
	return dst;
}

qFloat   * vec_cross(qFloat *v1, qFloat *v2, int len)
{
	/* cross products for i, j, k mean this makes sense
		for size 3 vectors only.  only do 0..2. */
	v1[0] = (v1[1] * v2[2] - v1[2] * v2[1]);
	v1[1] = (v1[2] * v2[0] - v1[0] * v2[2]);
	v1[2] = (v1[0] * v2[1] - v1[1] * v2[0]);

	return v1;
}

qFloat *cofactor_matrix(qFloat *cofmat, qFloat *srcmat,int i,int j , int n)
{
	/* eliminate row i and column j from matrix srcmat, put in cofmat */
	int skipi, skipj, ii, jj;

	for ( ii = 0 ; ii < n - 1 ; ii++) {
		skipi =  (ii >= i)? 1 : 0;
		for (jj = 0; jj < n - 1; jj++) {
			skipj =  (jj >= j)? 1 : 0;
			cofmat[QMidx(n-1, ii , jj )] = 
				srcmat[QMidx(n, ii + skipi, jj + skipj)];
		} /* for jj */
	}
	return cofmat;
}

qFloat cofactor(qFloat *cofmat, int i, int j, int n)
{
	qFloat cofactor_value;

	cofactor_value = (qFloat) (1 - 2 * (( i + j) % 2)) *
		determinant(cofmat, n);

	return cofactor_value;
}
	
qFloat *adjoint(qFloat *m, int n) {
	int i, j;
	qFloat *adj, *cm;

	adj = (qFloat *) malloc(sizeof(qFloat) * n * n);
	cm = (qFloat *) malloc(sizeof(qFloat) * (n-1) * (n-1));

	for (i = 0 ; i < n ; i++)
		for (j = 0 ; j < n ; j++) {
			cofactor_matrix(cm, m, i, j, n);
				adj[QMidx(n, j, i)] = cofactor(cm, i, j, n - 1);
		}
	free(cm);/*tidy*/
	return adj;
}

qFloat *minverse(qFloat *m, int n) {
	qFloat d, *inv;

	d = determinant(m, n);
	if (d == 0.00)
		return NULL;
	inv = adjoint(m, n);
	matrix_scalar_mul(1.0 / d, inv, n, n);
	return inv;
}

	

qFloat determinant(qFloat *m, int n) /*square matrices only */
{
	qFloat d;
	qFloat *mtmp;
	/* int skipon, j, ii; */
	int i;

	switch (n) {
	case 1:
		d = m[0];
		break;
	case 2:
		d = m[0] * m[3] - m[2] * m[1];
		break;
	case 3: d =	  m[Qm11] * m[Qm22] * m[Qm33] 
			+ m[Qm12] * m[Qm23] * m[Qm31] 
			+ m[Qm13] * m[Qm21] * m[Qm32] 
			- m[Qm11] * m[Qm23] * m[Qm32] 
			- m[Qm12] * m[Qm21] * m[Qm33] 
			- m[Qm13] * m[Qm22] * m[Qm31];
		break;
	default:  /* matrices of rank 4 and up */
		mtmp = (qFloat *) malloc(sizeof(qFloat) * (n-1) * (n-1));
		d = 0.0;	
		for ( i = 0 ; i < n ; i++) {
			/* create recursive determinant matrix */
			/* eliminate row j and column i */
			/* for us, eliminate row 0 */
			cofactor_matrix(mtmp, m, i, 0,n);
			/*printf("Cmatirx %d %d \n", i, 0);
			mprint(mtmp, n-1, n-1);
			printf("Cofactor is: %f of %d\n",
				cofactor(mtmp, i, 0, n - 1), n); */
			d += cofactor(mtmp, i, 0, n - 1) *
				m[QMidx(n, i, 0)];
		} /* for i */
		free(mtmp); /*tidy*/
		break;
	} /* switch */
	return d;
}

qFloat *matrix_scalar_mul(qFloat f, qFloat *mtarg, int m, int n){
	int  size, i;

	size = m * n;
	for (i = 0 ; i < size ; i++)
		mtarg[i] *= f;

	return mtarg;
}

qFloat   * matrix_mul(qFloat *m1, qFloat *m2, int m1m, int m1n, int m2m, int m2n)
{
	qFloat tmpf, *mxm;
	int w, h, i, j, s;


	if (m1m != m2n) return NULL;

	w = m2m;
	h = m1n;

	mxm = (qFloat *) malloc(sizeof(qFloat) * w * h);
	for (i = 0 ; i < w ; i++)
		for (j = 0 ; j < h ; j++) {
			tmpf = 0.0;
			/* take dot product of m1's ith row and m2's j'th column */
			for ( s = 0; s < m1m ; s++)
				tmpf +=
					m1[QMidx(m1m,s, j)] * 
					m2[QMidx(m2m,i, s)];
			mxm[QMidx(w, i, j)] = tmpf;
			
		} /*for j*/

	return mxm;
}
		

qFloat *mtranspose(qFloat *m, int w, int h)
{
	int i, j;
	qFloat tmpf;

	for( i = 0 ; i < w ; i++)
		for( j = 0 ; j < h ; j++) {
			tmpf = m[QMidx(w, i, j)];
			m[QMidx(w, i, j)] = m[QMidx(h, j, i)];
			m[QMidx(h, j, i)] = tmpf;
		}
	return m;
}

qFloat *mcopy_acc(qFloat *mdst, qFloat *msrc, int w, int h)
{
	int i, size;

	size = w * h;
	for (i = 0 ; i < size ; i++)
		mdst[i] = msrc[i];

	return mdst;
}

	
		
qFloat *mcopy(qFloat *msrc, int w, int h)
{
	qFloat *newm;

	newm = malloc(sizeof(qFloat) * w * h);
	mcopy_acc(newm, msrc, w, h);

	return newm;
}

	

void mprint(qFloat *m, int w, int h)
{
	int i, j;
	for (j = 0 ; j < h; j++) {
		printf("[ ");
		for( i = 0; i < w; i++)
			printf("%5.2f ", m[w * j + i]);
		printf("]\n");
	}
}

float deg2rad(float deg) {
        float rad;
                                                                                    
        /*rad = (deg / 360.0 * 3.141592654 * 2.0); */
        rad = deg * 0.0174532925222;
        return rad;
}

